@@ -40,10 +40,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#ifdef _GNU_SOURCE
-#error Make the crap stop. _GNU_SOURCE is completely unportable and breaks all sorts of behavior
-#endif
-
 #if !defined(__cplusplus) && (__GNUC__ > 2)
 #define GNUC_PRINTF(x, y) __attribute__((__format__(__printf__, x, y)))
 #else
@@ -95,6 +91,8 @@ char *       mkdtemp     (char *template);
 
 char *       strdup_path_mangle (const char *template);
 
+void         p11_dl_close       (void * dl);
+
 /* -----------------------------------------------------------------------------
  * WIN32
  */
@@ -151,8 +149,6 @@ typedef HMODULE dl_module_t;
 
 char *    p11_dl_error       (void);
 
-void      p11_dl_close       (void * dl);
-
 #define p11_sleep_ms(ms) \
 	(Sleep (ms))
 
@@ -210,8 +206,6 @@ typedef void * dl_module_t;
 
 #define p11_dl_open(f) \
 	(dlopen ((f), RTLD_LOCAL | RTLD_NOW))
-#define p11_dl_close \
-	dlclose
 #define p11_dl_symbol(d, s) \
 	(dlsym ((d), (s)))
 
@@ -264,6 +258,14 @@ char *     strndup          (const char *data,
 
 #endif /* HAVE_STRDUP */
 
+#ifndef HAVE_REALLOCARRAY
+
+void *     reallocarray     (void *ptr,
+                             size_t nmemb,
+                             size_t size);
+
+#endif /* HAVE_REALLOCARRAY */
+
 #ifdef HAVE_STDBOOL_H
 #include <stdbool.h>
 #else
@@ -338,4 +340,7 @@ int        fdwalk           (int (* cb) (void *data, int fd),
 
 #endif
 
+int        p11_ascii_tolower (int c);
+int        p11_ascii_toupper (int c);
+
 #endif /* __COMPAT_H__ */
