@@ -44,6 +44,9 @@
 #include "message.h"
 
 #include <assert.h>
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -59,12 +62,18 @@ static p11_local * _p11_library_get_thread_local (void);
 
 p11_mutex_t p11_library_mutex;
 
+p11_mutex_t p11_virtual_mutex;
+
 #ifdef OS_UNIX
 pthread_once_t p11_library_once = PTHREAD_ONCE_INIT;
 #endif
 
 unsigned int p11_forkid = 1;
 
+#ifdef HAVE_LOCALE_H
+extern locale_t p11_message_locale;
+#endif
+
 static char *
 thread_local_message (void)
 {
@@ -118,8 +127,12 @@ p11_library_init_impl (void)
 	p11_debug_init ();
 	p11_debug ("initializing library");
 	p11_mutex_init (&p11_library_mutex);
+	p11_mutex_init (&p11_virtual_mutex);
 	pthread_key_create (&thread_local, free);
 	p11_message_storage = thread_local_message;
+#ifdef HAVE_NEWLOCALE
+	p11_message_locale = newlocale (LC_ALL_MASK, "POSIX", (locale_t) 0);
+#endif
 
 	pthread_atfork (NULL, NULL, count_forks);
 }
@@ -139,8 +152,12 @@ p11_library_uninit (void)
 	free (pthread_getspecific (thread_local));
 	pthread_setspecific (thread_local, NULL);
 
+#ifdef HAVE_NEWLOCALE
+	freelocale (p11_message_locale);
+#endif
 	p11_message_storage = dont_store_message;
 	pthread_key_delete (thread_local);
+	p11_mutex_uninit (&p11_virtual_mutex);
 	p11_mutex_uninit (&p11_library_mutex);
 }
 
@@ -175,6 +192,7 @@ p11_library_init (void)
 	p11_debug_init ();
 	p11_debug ("initializing library");
 	p11_mutex_init (&p11_library_mutex);
+	p11_mutex_init (&p11_virtual_mutex);
 	thread_local = TlsAlloc ();
 	if (thread_local == TLS_OUT_OF_INDEXES)
 		p11_debug ("couldn't setup tls");
@@ -206,6 +224,7 @@ p11_library_uninit (void)
 		LocalFree (data);
 		TlsFree (thread_local);
 	}
+	p11_mutex_uninit (&p11_virtual_mutex);
 	p11_mutex_uninit (&p11_library_mutex);
 }
 
